/* 	Cory McWilliams
		mcwilc@rpi.edu
		9/14/2003					*/

#if defined (__APPLE__)
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>

#define ROCKCOUNT 10
#define ABS(x) (((x)>0)?(x):(-(x)))

float rocks[ROCKCOUNT];
float yvel;
float ypos;
float crash;
int crashes;
time_t start;
float spin;

// Pick a place for a new rock somewhere below
float newrock() {
	float n = (((double(rand())) / RAND_MAX) * 1.0f + 2.0f);
	bool notsafe = true;
	while (notsafe) {
		notsafe = false;
		for (int i = 0; i < ROCKCOUNT; i++) {
			if (ABS(rocks[i] - n) < 0.4f)
				notsafe = true;
		}

		if (notsafe) n += 1.0f;
	}
	return n;
}

// Set up the background color...not used, the environment, and the random number generator
void init() {
	srand(time(0));
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	crashes = 0;
	
	for (int i = 0; i < ROCKCOUNT; i++)
		rocks[i] = newrock();
}

// Try to keep the display square.  This should look good, but you can see the background if the window gets too thin
void reshape(int w, int h) {
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	float s = float(w) / h * 1.0f;
	gluOrtho2D(-s, s, -1.0f, 1.0f);
}

// Draw a wagon wheel, currently a shady circle with white squares in the center
void wheel(float x, float y, float r, int v) {
	glPushMatrix();
	glTranslatef(x, y, 0.0f);
	glRotatef(-spin, 0.0f, 0.0f, 1.0f);
	glBegin(GL_POLYGON);
	for (int i = 0; i < v; i++) {
		if ((i % (v/2)) == 0) glColor3f(0.5f, 0.5f, 0.5f);
		else glColor3f(0.0f, 0.0f, 0.0f);
		glVertex2f(float(r * cos(2.0 * 3.1415926535 / v * i)), float(r * sin(2.0 * 3.1415926535 / v * i)));
	}
	glEnd();
	glColor3f(1.0f, 1.0f, 1.0f);
	glBegin(GL_QUADS);
	glVertex2f(r / 3, -r / 3);
	glVertex2f(r / 3, r / 3);
	glVertex2f(-r / 3, r / 3);
	glVertex2f(-r / 3, -r / 3);
	glEnd();
	glPopMatrix();
}

void display() {
	glClear(GL_COLOR_BUFFER_BIT);

	glPushMatrix();
	glRotatef(-45.0f, 0.0f, 0.0f, 1.0f); // A steep hill :)
	
	// I know there's lots of unnecessary "overspray"...it made it easier to not think about rotating it and...
	// Draw the ground
	glColor3f(0.0f, 1.0f, 0.0f);
	glBegin(GL_QUADS);
	glVertex2f(2.0f, -2.0f);
	glVertex2f(2.0f, 0.0f);
	glVertex2f(-2.0f, 0.0f);
	glColor3f(0.0f, 0.25f, 0.0f); // I love OpenGL gradients...
	glVertex2f(-2.0f, -2.0f);
	
	// Draw the sky
	glColor3f(0.0f, 0.0f, 1.0f);
	glVertex2f(2.0f, 0.0f);
	glVertex2f(2.0f, 2.0f);
	glColor3f(0.0f, 0.0f, 0.25f); // I love OpenGL gradients...
	glVertex2f(-2.0f, 2.0f);
	glVertex2f(-2.0f, 0.0f);
	glEnd();
	
	for (int i = 0; i < ROCKCOUNT; i++) {
		if (rocks[i] >= -2.0f && rocks[i] <= 2.0f) {
			glPushMatrix();

			// Draw a rock
			glTranslatef(rocks[i], 0.05f, 0.0f);
			glColor3f(0.5f, 0.5f, 0.5f);
			glBegin(GL_POLYGON);
			glVertex2f(0.05f, -0.1f);
			glVertex2f(0.09f, -0.05f);
			glVertex2f(0.05f, 0.03f);
			glVertex2f(-0.05f, 0.03f);
			glColor3f(0.25f, 0.25f, 0.25f);
			glVertex2f(-0.09f, -0.05f);
			glVertex2f(-0.05f, -0.1f);
			glEnd();

			glPopMatrix();
		}
	}

	// Draw the wagon
	glPushMatrix();

	glTranslatef(0.0f, ypos, 0.0f);
	if (crash > 0) {
		glRotatef((float(rand()) / RAND_MAX - 0.5f) * 30.0f, 0.0f, 0.0f, 1.0f); // Make it bounce around a bit.
	}

	// A handle dangling off the back
	glPushMatrix();
	glColor3f(0.0f, 0.0f, 0.0f);
	glTranslatef(-0.1f, 0.079f, 0.0f);
	glRotatef(yvel * 15.0f, 0.0f, 0.0f, 1.0f);
	glRectf(-0.15f, -0.005f, 0.0f, 0.005f);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0.0f, ypos / 6, 0.0f);
	// Calvin, or an abstract outline of him... XD
	glColor3f(1.0f, 1.0f, 0.0f);
	glBegin(GL_POLYGON);
	glVertex2f(0.08f, 0.075f);
	glVertex2f(0.08f, 0.25f);
	glVertex2f(0.01f, 0.25f);
	glVertex2f(0.01f, 0.075f);
	glEnd();

	// And Calvin's stuffed tiger, Hobbes.
	glColor3f(1.0f, 0.25f, 0.0f);
	glBegin(GL_POLYGON);
	glVertex2f(-0.01f, 0.075f);
	glVertex2f(-0.01f, 0.2f);
	glVertex2f(-0.04f, 0.2f);
	glVertex2f(-0.04f, 0.075f);
	glEnd();
	glPopMatrix();

	// The red part
	glBegin(GL_QUADS);
	glColor3f(1.0f, 0.0f, 0.0f);
	glVertex2f(0.1f, 0.05f);
	glVertex2f(0.1f, 0.1f);
	glVertex2f(-0.1f, 0.1f);
	glColor3f(0.8f, 0.0f, 0.0f);
	glVertex2f(-0.1f, 0.05f);
	glEnd();

	// Some wheels
	glColor3f(0.0f, 0.0f, 0.0f);
	wheel(-0.07f, 0.04f, 0.04f, 10);
	wheel(0.07f, 0.04f, 0.04f, 10);

	glPopMatrix();

	glPopMatrix();
	
	glutSwapBuffers();
}

// Affect animations here
void timer(int value) {
	for (int i = 0; i < ROCKCOUNT; i++) {
		if ((rocks[i] -= 0.01f) < -2.0f) {
			rocks[i] = newrock();
		} else if (ABS(rocks[i]) < 0.1f && ypos < 0.05f && crash < 0) {
			crash = 1.0f;
			crashes++; // Ok...this number isn't really the number of rocks, but it's fun.
		}
	}

	if (crash >= 0) crash -= 0.1f;
	yvel -= 0.1f; // gravity
	ypos += yvel * 0.01f; // vel * time
	if (ypos < 0) { ypos = 0; yvel = 0; }
	spin += 10.0f;
	if (spin > 360.0f) spin -= 360.0f;
	glutTimerFunc(10, timer, 0);
	glutPostRedisplay();
	glutReportErrors();
}

// This is probably the simplest key handler I will ever write
void key(unsigned char key, int x, int y) {
	if (yvel <= 0 && ypos <= 0) yvel = 3.0f;
}

// Any joystick buttons -> a button was pressed.
void joystickFunc(unsigned int buttons, int x, int y, int z) {
	if (buttons) {
		key('j', 0, 0);
	}
}

// Conclusion...set up by atexit
void finish() {
	std::cout << "Thank you for playing.\n";
	std::cout << "You received " << crashes << " damage in " << time(0) - start << " seconds.\n";
}

// Entry point
int main(int argc, char *argv[]) {
	printf("Cory McWilliams\nmcwilc@rpi.edu\n9/14/2003\n\n");
	start = time(0);
	atexit(finish);
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGBA);
	glutInitWindowSize(800, 600);
	glutCreateWindow(argv[0]);
	glutJoystickFunc(joystickFunc, 10);
	glutFullScreen();
	glutSetCursor(GLUT_CURSOR_NONE);
	init();
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	glutTimerFunc(1, timer, 0);
	glutKeyboardFunc(key);
	glutMainLoop();
	return 0;
}
